@@ -1,5 +1,7 @@
 # Copyright (c) 2003-2010 Sylvain Thenault (thenault@gmail.com).
-# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
+# Copyright 2012 Google Inc.
+#
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
@@ -31,49 +33,61 @@ from logilab.astng import nodes
 
 from pylint.interfaces import IRawChecker, IASTNGChecker
 from pylint.checkers import BaseRawChecker
-from pylint.checkers.misc import guess_encoding, is_ascii
+from pylint.checkers.utils import check_messages
 
 MSGS = {
     'C0301': ('Line too long (%s/%s)',
+              'line-too-long',
               'Used when a line is longer than a given number of characters.'),
     'C0302': ('Too many lines in module (%s)', # was W0302
+              'too-many-lines',
               'Used when a module has too much lines, reducing its readability.'
               ),
 
     'W0311': ('Bad indentation. Found %s %s, expected %s',
+              'bad-indentation',
               'Used when an unexpected number of indentation\'s tabulations or '
               'spaces has been found.'),
     'W0312': ('Found indentation with %ss instead of %ss',
+              'mixed-indentation',
               'Used when there are some mixed tabs and spaces in a module.'),
     'W0301': ('Unnecessary semicolon', # was W0106
+              'unnecessary-semicolon',
               'Used when a statement is ended by a semi-colon (";"), which \
               isn\'t necessary (that\'s python, not C ;).'),
-
-    'F0321': ('Format detection error in %r',
-              'Used when an unexpected error occurred in bad format detection.'
-              'Please report the error if it occurs.'),
     'C0321': ('More than one statement on a single line',
+              'multiple-statements',
               'Used when more than on statement are found on the same line.'),
     'C0322': ('Operator not preceded by a space\n%s',
+              'no-space-before-operator',
               'Used when one of the following operator (!= | <= | == | >= | < '
-              '| > | = | \+= | -= | \*= | /= | %) is not preceded by a space.'),
+              '| > | = | \\+= | -= | \\*= | /= | %) is not preceded by a space.'),
     'C0323': ('Operator not followed by a space\n%s',
+              'no-space-after-operator',
               'Used when one of the following operator (!= | <= | == | >= | < '
-              '| > | = | \+= | -= | \*= | /= | %) is not followed by a space.'),
+              '| > | = | \\+= | -= | \\*= | /= | %) is not followed by a space.'),
     'C0324': ('Comma not followed by a space\n%s',
+              'no-space-after-comma',
               'Used when a comma (",") is not followed by a space.'),
+    }
 
+if sys.version_info < (3, 0):
+
+    MSGS.update({
     'W0331': ('Use of the <> operator',
+              'old-ne-operator',
               'Used when the deprecated "<>" operator is used instead \
               of "!=".'),
-    'W0332': ('Use l as long integer identifier',
+    'W0332': ('Use of "l" as long integer identifier',
+              'lowercase-l-suffix',
               'Used when a lower case "l" is used to mark a long integer. You '
               'should use a upper case "L" since the letter "l" looks too much '
               'like the digit "1"'),
     'W0333': ('Use of the `` operator',
+              'backtick',
               'Used when the deprecated "``" (backtick) operator is used '
               'instead  of the str() function.'),
-    }
+    })
 
 # simple quoted string rgx
 SQSTRING_RGX = r'"([^"\\]|\\.)*?"'
@@ -81,7 +95,7 @@ SQSTRING_RGX = r'"([^"\\]|\\.)*?"'
 SASTRING_RGX = r"'([^'\\]|\\.)*?'"
 # triple quoted string rgx
 TQSTRING_RGX = r'"""([^"]|("(?!"")))*?(""")'
-# triple apostrophed string rgx # FIXME english please
+# triple apostrophe'd string rgx
 TASTRING_RGX = r"'''([^']|('(?!'')))*?(''')"
 
 # finally, the string regular expression
@@ -109,11 +123,12 @@ BAD_CONSTRUCT_RGXS = (
      re.compile(OP_RGX_SEARCH_2, re.M),
      'C0323'),
 
-    (re.compile(r'.*,[^(\s|\]|}|\))].*', re.M), 
+    (re.compile(r'.*,[^(\s|\]|}|\))].*', re.M),
      re.compile(r',[^\s)]', re.M),
      'C0324'),
     )
 
+_PY3K = sys.version_info >= (3, 0)
 
 def get_string_coords(line):
     """return a list of string positions (tuple (start, end)) in the line
@@ -131,7 +146,7 @@ def in_coords(match, string_coords):
             return True
     return False
 
-def check_line(line, writer):
+def check_line(line):
     """check a line for a bad construction
     if it founds one, return a message describing the problem
     else return None
@@ -143,7 +158,6 @@ def check_line(line, writer):
             for match in re.finditer(rgx_search, line):
                 if not in_coords(match, string_positions):
                     return msg_id, pretty_match(match, line.rstrip())
-            #writer.add_message('F0321', line=line, args=line)
 
 
 class FormatChecker(BaseRawChecker):
@@ -179,22 +193,17 @@ class FormatChecker(BaseRawChecker):
         self._lines = None
         self._visited_lines = None
 
-    def process_module(self, stream):
+    def process_module(self, node):
         """extracts encoding from the stream and decodes each line, so that
         international text's length is properly calculated.
         """
-        line_reader = stream.readline
+        stream = node.file_stream
+        stream.seek(0) # XXX may be removed with astng > 0.23
+        readline = stream.readline
         if sys.version_info < (3, 0):
-            data = stream.read()
-            if not is_ascii(data)[0]:
-                encoding = guess_encoding(data)
-                if encoding is not None:
-                    line_reader = lambda: stream.readline().decode(encoding,
-                                                                   'replace')
-            del data
-
-        stream.seek(0)
-        self.process_tokens(tokenize.generate_tokens(line_reader))
+            if node.file_encoding is not None:
+                readline = lambda: stream.readline().decode(node.file_encoding, 'replace')
+        self.process_tokens(tokenize.generate_tokens(readline))
 
     def new_line(self, tok_type, line, line_num, junk):
         """a new line has been encountered, process it if necessary"""
@@ -274,6 +283,7 @@ class FormatChecker(BaseRawChecker):
         if line_num > self.config.max_module_lines:
             self.add_message('C0302', args=line_num, line=1)
 
+    @check_messages('C0321' ,'C03232', 'C0323', 'C0324')
     def visit_default(self, node):
         """check the node line number and check it if not yet done"""
         if not node.is_statement:
@@ -310,13 +320,14 @@ class FormatChecker(BaseRawChecker):
             except KeyError:
                 lines.append('')
         try:
-            msg_def = check_line('\n'.join(lines), self)
+            msg_def = check_line('\n'.join(lines))
             if msg_def:
                 self.add_message(msg_def[0], node=node, args=msg_def[1])
         except KeyError:
             # FIXME: internal error !
             pass
 
+    @check_messages('W0333')
     def visit_backquote(self, node):
         self.add_message('W0333', node=node)
 
@@ -360,6 +371,103 @@ class FormatChecker(BaseRawChecker):
                                    expected * unit_size))
 
 
+class StringConstantChecker(BaseRawChecker):
+    """Check string literals"""
+
+    msgs = {
+        'W1401': ('Anomalous backslash in string: \'%s\'. '
+                  'String constant might be missing an r prefix.',
+                  'anomalous-backslash-in-string',
+                  'Used when a backslash is in a literal string but not as an '
+                  'escape.'),
+        'W1402': ('Anomalous Unicode escape in byte string: \'%s\'. '
+                  'String constant might be missing an r or u prefix.',
+                  'anomalous-unicode-escape-in-string',
+                  'Used when an escape like \\u is encountered in a byte '
+                  'string where it has no effect.'),
+        }
+    name = 'string_constant'
+    __implements__ = (IRawChecker, IASTNGChecker)
+
+    # Characters that have a special meaning after a backslash in either
+    # Unicode or byte strings.
+    ESCAPE_CHARACTERS = 'abfnrtvox\n\r\t\\\'\"'
+
+    # Characters that have a special meaning after a backslash but only in
+    # Unicode strings.
+    UNICODE_ESCAPE_CHARACTERS = 'uUN'
+
+    def process_tokens(self, tokens):
+        for (tok_type, token, (start_row, start_col), _, _) in tokens:
+            if tok_type == tokenize.STRING:
+                # 'token' is the whole un-parsed token; we can look at the start
+                # of it to see whether it's a raw or unicode string etc.
+                self.process_string_token(token, start_row, start_col)
+
+    def process_string_token(self, token, start_row, start_col):
+        for i, c in enumerate(token):
+            if c in '\'\"':
+                quote_char = c
+                break
+        prefix = token[:i].lower()  #  markers like u, b, r.
+        after_prefix = token[i:]
+        if after_prefix[:3] == after_prefix[-3:] == 3 * quote_char:
+            string_body = after_prefix[3:-3]
+        else:
+            string_body = after_prefix[1:-1]  # Chop off quotes
+        # No special checks on raw strings at the moment.
+        if 'r' not in prefix:
+            self.process_non_raw_string_token(prefix, string_body,
+                start_row, start_col)
+
+    def process_non_raw_string_token(self, prefix, string_body, start_row,
+                                     start_col):
+        """check for bad escapes in a non-raw string.
+
+        prefix: lowercase string of eg 'ur' string prefix markers.
+        string_body: the un-parsed body of the string, not including the quote
+        marks.
+        start_row: integer line number in the source.
+        start_col: integer column number in the source.
+        """
+        # Walk through the string; if we see a backslash then escape the next
+        # character, and skip over it.  If we see a non-escaped character,
+        # alert, and continue.
+        #
+        # Accept a backslash when it escapes a backslash, or a quote, or
+        # end-of-line, or one of the letters that introduce a special escape
+        # sequence <http://docs.python.org/reference/lexical_analysis.html>
+        #
+        # TODO(mbp): Maybe give a separate warning about the rarely-used
+        # \a \b \v \f?
+        #
+        # TODO(mbp): We could give the column of the problem character, but
+        # add_message doesn't seem to have a way to pass it through at present.
+        i = 0
+        while True:
+            i = string_body.find('\\', i)
+            if i == -1:
+                break
+            # There must be a next character; having a backslash at the end
+            # of the string would be a SyntaxError.
+            next_char = string_body[i+1]
+            match = string_body[i:i+2]
+            if next_char in self.UNICODE_ESCAPE_CHARACTERS:
+                if 'u' in prefix:
+                    pass
+                elif _PY3K and 'b' not in prefix:
+                    pass  # unicode by default
+                else:
+                    self.add_message('W1402', line=start_row, args=(match, ))
+            elif next_char not in self.ESCAPE_CHARACTERS:
+                self.add_message('W1401', line=start_row, args=(match, ))
+            # Whether it was a valid escape or not, backslash followed by
+            # another character can always be consumed whole: the second
+            # character can never be the start of a new backslash escape.
+            i += 2
+
+
 def register(linter):
     """required method to auto register this checker """
     linter.register_checker(FormatChecker(linter))
+    linter.register_checker(StringConstantChecker(linter))
