@@ -13,126 +13,58 @@
 # You should have received a copy of the GNU General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-
-
 """Checker for string formatting operations.
 """
 
-import string
 from logilab import astng
 from pylint.interfaces import IASTNGChecker
 from pylint.checkers import BaseChecker
+from pylint.checkers import utils
+
 
 MSGS = {
-    'E9900': ("Unsupported format character %r (%#02x) at index %d",
+    'E1300': ("Unsupported format character %r (%#02x) at index %d",
+              "bad-format-character",
               "Used when a unsupported format character is used in a format\
               string."),
-    'E9901': ("Format string ends in middle of conversion specifier",
+    'E1301': ("Format string ends in middle of conversion specifier",
+              "truncated-format-string",
               "Used when a format string terminates before the end of a \
               conversion specifier."),
-    'E9902': ("Mixing named and unnamed conversion specifiers in format string",
+    'E1302': ("Mixing named and unnamed conversion specifiers in format string",
+              "mixed-format-string",
               "Used when a format string contains both named (e.g. '%(foo)d') \
               and unnamed (e.g. '%d') conversion specifiers.  This is also \
               used when a named conversion specifier contains * for the \
               minimum field width and/or precision."),
-    'E9903': ("Expected mapping for format string, not %s",
+    'E1303': ("Expected mapping for format string, not %s",
+              "format-needs-mapping",
               "Used when a format string that uses named conversion specifiers \
               is used with an argument that is not a mapping."),
-    'W9900': ("Format string dictionary key should be a string, not %s",
+    'W1300': ("Format string dictionary key should be a string, not %s",
+              "bad-format-string-key",
               "Used when a format string that uses named conversion specifiers \
               is used with a dictionary whose keys are not all strings."),
-    'W9901': ("Unused key %r in format string dictionary",
+    'W1301': ("Unused key %r in format string dictionary",
+              "unused-format-string-key",
               "Used when a format string that uses named conversion specifiers \
               is used with a dictionary that conWtains keys not required by the \
               format string."),
-    'E9904': ("Missing key %r in format string dictionary",
+    'E1304': ("Missing key %r in format string dictionary",
+              "missing-format-string-key",
               "Used when a format string that uses named conversion specifiers \
               is used with a dictionary that doesn't contain all the keys \
               required by the format string."),
-    'E9905': ("Too many arguments for format string",
+    'E1305': ("Too many arguments for format string",
+              "too-many-format-args",
               "Used when a format string that uses unnamed conversion \
               specifiers is given too few arguments."),
-    'E9906': ("Not enough arguments for format string",
+    'E1306': ("Not enough arguments for format string",
+              "too-few-format-args",
               "Used when a format string that uses unnamed conversion \
               specifiers is given too many arguments"),
     }
 
-class IncompleteFormatString(Exception):
-    """A format string ended in the middle of a format specifier."""
-    pass
-
-class UnsupportedFormatCharacter(Exception):
-    """A format character in a format string is not one of the supported
-    format characters."""
-    def __init__(self, index):
-        Exception.__init__(self, index)
-        self.index = index
-
-def parse_format_string(format_string):
-    """Parses a format string, returning a tuple of (keys, num_args), where keys
-    is the set of mapping keys in the format string, and num_args is the number
-    of arguments required by the format string.  Raises
-    IncompleteFormatString or UnsupportedFormatCharacter if a
-    parse error occurs."""
-    keys = set()
-    num_args = 0
-    def next_char(i):
-        i += 1
-        if i == len(format_string):
-            raise IncompleteFormatString
-        return (i, format_string[i])
-    i = 0
-    while i < len(format_string):
-        c = format_string[i]
-        if c == '%':
-            i, c = next_char(i)
-            # Parse the mapping key (optional).
-            key = None
-            if c == '(':
-                depth = 1
-                i, c = next_char(i)
-                key_start = i
-                while depth != 0:
-                    if c == '(':
-                        depth += 1
-                    elif c == ')':
-                        depth -= 1
-                    i, c = next_char(i)
-                key_end = i - 1
-                key = format_string[key_start:key_end]
-
-            # Parse the conversion flags (optional).
-            while c in '#0- +':
-                i, c = next_char(i)
-            # Parse the minimum field width (optional).
-            if c == '*':
-                num_args += 1
-                i, c = next_char(i)
-            else:
-                while c in string.digits:
-                    i, c = next_char(i)
-            # Parse the precision (optional).
-            if c == '.':
-                i, c = next_char(i)
-                if c == '*':
-                    num_args += 1
-                    i, c = next_char(i)
-                else:
-                    while c in string.digits:
-                        i, c = next_char(i)
-            # Parse the length modifier (optional).
-            if c in 'hlL':
-                i, c = next_char(i)
-            # Parse the conversion type (mandatory).
-            if c not in 'diouxXeEfFgGcrs%':
-                raise UnsupportedFormatCharacter(i)
-            if key:
-                keys.add(key)
-            elif c != '%':
-                num_args += 1
-        i += 1
-    return keys, num_args
-
 OTHER_NODES = (astng.Const, astng.List, astng.Backquote,
                astng.Lambda, astng.Function,
                astng.ListComp, astng.SetComp, astng.GenExpr)
@@ -158,18 +90,18 @@ class StringFormatChecker(BaseChecker):
         format_string = left.value
         try:
             required_keys, required_num_args = \
-                parse_format_string(format_string)
-        except UnsupportedFormatCharacter, e:
+                utils.parse_format_string(format_string)
+        except utils.UnsupportedFormatCharacter, e:
             c = format_string[e.index]
-            self.add_message('E9900', node=node, args=(c, ord(c), e.index))
+            self.add_message('E1300', node=node, args=(c, ord(c), e.index))
             return
-        except IncompleteFormatString:
-            self.add_message('E9901', node=node)
+        except utils.IncompleteFormatString:
+            self.add_message('E1301', node=node)
             return
         if required_keys and required_num_args:
             # The format string uses both named and unnamed format
             # specifiers.
-            self.add_message('E9902', node=node)
+            self.add_message('E1302', node=node)
         elif required_keys:
             # The format string uses only named format specifiers.
             # Check that the RHS of the % operator is a mapping object
@@ -178,13 +110,13 @@ class StringFormatChecker(BaseChecker):
             if isinstance(args, astng.Dict):
                 keys = set()
                 unknown_keys = False
-                for k, v in args.items:
+                for k, _ in args.items:
                     if isinstance(k, astng.Const):
                         key = k.value
                         if isinstance(key, basestring):
                             keys.add(key)
                         else:
-                            self.add_message('W9900', node=node, args=key)
+                            self.add_message('W1300', node=node, args=key)
                     else:
                         # One of the keys was something other than a
                         # constant.  Since we can't tell what it is,
@@ -194,13 +126,13 @@ class StringFormatChecker(BaseChecker):
                 if not unknown_keys:
                     for key in required_keys:
                         if key not in keys:
-                            self.add_message('E9904', node=node, args=key)
+                            self.add_message('E1304', node=node, args=key)
                 for key in keys:
                     if key not in required_keys:
-                        self.add_message('W9901', node=node, args=key)
+                        self.add_message('W1301', node=node, args=key)
             elif isinstance(args, OTHER_NODES + (astng.Tuple,)):
                 type_name = type(args).__name__
-                self.add_message('E9903', node=node, args=type_name)
+                self.add_message('E1303', node=node, args=type_name)
             # else:
                 # The RHS of the format specifier is a name or
                 # expression.  It may be a mapping object, so
@@ -221,9 +153,9 @@ class StringFormatChecker(BaseChecker):
                 num_args = None
             if num_args is not None:
                 if num_args > required_num_args:
-                    self.add_message('E9905', node=node)
+                    self.add_message('E1305', node=node)
                 elif num_args < required_num_args:
-                    self.add_message('E9906', node=node)
+                    self.add_message('E1306', node=node)
 
 
 def register(linter):
