@@ -16,6 +16,9 @@
 """try to find more bugs in the code using astng inference capabilities
 """
 
+import re
+import shlex
+
 from logilab import astng
 from logilab.astng import InferenceError, NotFoundError, YES, Instance
 
@@ -25,33 +28,43 @@ from pylint.checkers.utils import safe_infer, is_super, check_messages
 
 MSGS = {
     'E1101': ('%s %r has no %r member',
+              'no-member',
               'Used when a variable is accessed for an unexistent member.'),
     'E1102': ('%s is not callable',
+              'not-callable',
               'Used when an object being called has been inferred to a non \
               callable object'),
     'E1103': ('%s %r has no %r member (but some types could not be inferred)',
+              'maybe-no-member',
               'Used when a variable is accessed for an unexistent member, but \
               astng was not able to interpret all possible types of this \
               variable.'),
     'E1111': ('Assigning to function call which doesn\'t return',
+              'assignment-from-no-return',
               'Used when an assignment is done on a function call but the \
               inferred function doesn\'t return anything.'),
     'W1111': ('Assigning to function call which only returns None',
+              'assignment-from-none',
               'Used when an assignment is done on a function call but the \
               inferred function returns nothing but None.'),
 
     'E1120': ('No value passed for parameter %s in function call',
+              'no-value-for-parameter',
               'Used when a function call passes too few arguments.'),
     'E1121': ('Too many positional arguments for function call',
+              'too-many-function-args',
               'Used when a function call passes too many positional \
               arguments.'),
     'E1122': ('Duplicate keyword argument %r in function call',
+              'duplicate-keyword-arg',
               'Used when a function call passes the same keyword argument \
               multiple times.'),
     'E1123': ('Passing unexpected keyword argument %r in function call',
+              'unexpected-keyword-arg',
               'Used when a function call passes a keyword argument that \
               doesn\'t correspond to one of the function\'s parameter names.'),
     'E1124': ('Multiple values passed for parameter %r in function call',
+              'redundant-keyword-arg',
               'Used when a function call would result in assigning multiple \
               values to a function parameter, one value from a positional \
               argument and one from a keyword argument.'),
@@ -92,11 +105,11 @@ of Zope acquired attributes to generated-members.'}
                ('generated-members',
                 {'default' : (
         'REQUEST', 'acl_users', 'aq_parent'),
-                 'type' : 'csv',
+                 'type' : 'string',
                  'metavar' : '<members names>',
                  'help' : 'List of members which are set dynamically and \
 missed by pylint inference system, and so shouldn\'t trigger E0201 when \
-accessed.'}
+accessed. Python regular expressions are accepted.'}
                 ),
         )
 
@@ -122,9 +135,19 @@ accessed.'}
 
         function/method, super call and metaclasses are ignored
         """
-        if node.attrname in self.generated_members:
+        # generated_members may containt regular expressions
+        # (surrounded by quote `"` and followed by a comma `,`)
+        # REQUEST,aq_parent,"[a-zA-Z]+_set{1,2}"' =>
+        # ('REQUEST', 'aq_parent', '[a-zA-Z]+_set{1,2}')
+        if isinstance(self.config.generated_members, str):
+            gen = shlex.shlex(self.config.generated_members)
+            gen.whitespace += ','
+            gen.wordchars += '[]-+'
+            self.config.generated_members = tuple(tok.strip('"') for tok in gen)
+        for pattern in self.config.generated_members:
             # attribute is marked as generated, stop here
-            return
+            if re.match(pattern, node.attrname):
+                return
         try:
             infered = list(node.expr.infer())
         except InferenceError:
@@ -158,6 +181,8 @@ accessed.'}
                 # XXX method / function
                 continue
             except NotFoundError:
+                if isinstance(owner, astng.Function) and owner.decorators:
+                    continue
                 if isinstance(owner, Instance) and owner.has_dynamic_getattr():
                     continue
                 # explicit skipping of optparse'Values class
@@ -352,7 +377,7 @@ accessed.'}
         for [(name, defval), assigned] in parameters:
             if (defval is None) and not assigned:
                 if name is None:
-                    display = '<tuple>'
+                    display_name = '<tuple>'
                 else:
                     display_name = repr(name)
                 self.add_message('E1120', node=node, args=display_name)
