@@ -1,3 +1,18 @@
+# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
+# http://www.logilab.fr/ -- mailto:contact@logilab.fr
+#
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU General Public License as published by the Free Software
+# Foundation; either version 2 of the License, or (at your option) any later
+# version.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc.,
+# 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """Tkinker gui for pylint"""
 
 import os
@@ -5,8 +20,10 @@ import sys
 import re
 import Queue
 from threading import Thread
-from Tkinter import Tk, Frame, Listbox, Entry, Label, Button, Scrollbar, Checkbutton, Radiobutton, IntVar, StringVar
-from Tkinter import TOP, LEFT, RIGHT, BOTTOM, END, X, Y, BOTH, SUNKEN, W, HORIZONTAL, DISABLED, NORMAL, W, E
+from Tkinter import (Tk, Frame, Listbox, Entry, Label, Button, Scrollbar,
+                     Checkbutton, Radiobutton, IntVar, StringVar)
+from Tkinter import (TOP, LEFT, RIGHT, BOTTOM, END, X, Y, BOTH, SUNKEN, W,
+                     HORIZONTAL, DISABLED, NORMAL, W, E)
 from tkFileDialog import askopenfilename, askdirectory
 
 import pylint.lint
@@ -19,6 +36,14 @@ COLORS = {'(I)':'lightblue',
           '(W)':'black', '(E)':'darkred',
           '(F)':'red'}
 
+
+def convert_to_string(msg):
+    """make a string representation of a message"""
+    if (msg[2] != ""):
+        return "(" + msg[0] + ") " + msg[1] + "." + msg[2] + " [" + msg[3] + "]: " + msg[4]
+    else:
+        return "(" + msg[0] + ") " + msg[1] + " [" + msg[3] + "]: " + msg[4]
+
 class BasicStream:
     '''
     used in gui reporter instead of writing to stdout, it is written to
@@ -47,18 +72,21 @@ class BasicStream:
 
         if text.startswith('\n'):
             self.contents.append('')
-            if self.currout: self.outdict[self.currout].append('')
+            if self.currout:
+                self.outdict[self.currout].append('')
         self.contents[-1] += text.strip('\n')
-        if self.currout: self.outdict[self.currout][-1] += text.strip('\n')
+        if self.currout:
+            self.outdict[self.currout][-1] += text.strip('\n')
         if text.endswith('\n') and text.strip():
             self.contents.append('')
-            if self.currout: self.outdict[self.currout].append('')
+            if self.currout:
+                self.outdict[self.currout].append('')
 
     def fix_contents(self):
         """finalize what the contents of the dict should look like before output"""
         for item in self.outdict:
             numEmpty = self.outdict[item].count('')
-            for i in range(numEmpty):
+            for i in xrange(numEmpty):
                 self.outdict[item].remove('')
             if self.outdict[item]:
                 self.outdict[item].pop(0)
@@ -194,7 +222,8 @@ class LintGui:
 
         #buttons
         Button(top_frame, text='Open', command=self.file_open).pack(side=LEFT)
-        Button(top_frame, text='Open Package', command=(lambda : self.file_open(package=True))).pack(side=LEFT)
+        Button(top_frame, text='Open Package', 
+               command=(lambda : self.file_open(package=True))).pack(side=LEFT)
 
         self.btnRun = Button(top_frame, text='Run', command=self.run_lint)
         self.btnRun.pack(side=LEFT)
@@ -207,12 +236,18 @@ class LintGui:
         self.warning_box = IntVar()
         self.error_box = IntVar()
         self.fatal_box = IntVar()
-        i = Checkbutton(check_frame, text="Information", fg=COLORS['(I)'], variable=self.information_box, command=self.refresh_msg_window)
-        c = Checkbutton(check_frame, text="Convention", fg=COLORS['(C)'], variable=self.convention_box, command=self.refresh_msg_window)
-        r = Checkbutton(check_frame, text="Refactor", fg=COLORS['(R)'], variable=self.refactor_box, command=self.refresh_msg_window)
-        w = Checkbutton(check_frame, text="Warning", fg=COLORS['(W)'], variable=self.warning_box, command=self.refresh_msg_window)
-        e = Checkbutton(check_frame, text="Error", fg=COLORS['(E)'], variable=self.error_box, command=self.refresh_msg_window)
-        f = Checkbutton(check_frame, text="Fatal", fg=COLORS['(F)'], variable=self.fatal_box, command=self.refresh_msg_window)
+        i = Checkbutton(check_frame, text="Information", fg=COLORS['(I)'],
+                        variable=self.information_box, command=self.refresh_msg_window)
+        c = Checkbutton(check_frame, text="Convention", fg=COLORS['(C)'],
+                        variable=self.convention_box, command=self.refresh_msg_window)
+        r = Checkbutton(check_frame, text="Refactor", fg=COLORS['(R)'],
+                        variable=self.refactor_box, command=self.refresh_msg_window)
+        w = Checkbutton(check_frame, text="Warning", fg=COLORS['(W)'],
+                        variable=self.warning_box, command=self.refresh_msg_window)
+        e = Checkbutton(check_frame, text="Error", fg=COLORS['(E)'],
+                        variable=self.error_box, command=self.refresh_msg_window)
+        f = Checkbutton(check_frame, text="Fatal", fg=COLORS['(F)'],
+                        variable=self.fatal_box, command=self.refresh_msg_window)
         i.select()
         c.select()
         r.select()
@@ -229,13 +264,23 @@ class LintGui:
         #check boxes
         self.box = StringVar()
         # XXX should be generated
-        report = Radiobutton(radio_frame, text="Report", variable=self.box, value="Report", command=self.refresh_results_window)
-        rawMet = Radiobutton(radio_frame, text="Raw metrics", variable=self.box, value="Raw metrics", command=self.refresh_results_window)
-        dup = Radiobutton(radio_frame, text="Duplication", variable=self.box, value="Duplication", command=self.refresh_results_window)
-        ext = Radiobutton(radio_frame, text="External dependencies", variable=self.box, value="External dependencies", command=self.refresh_results_window)
-        stat = Radiobutton(radio_frame, text="Statistics by type", variable=self.box, value="Statistics by type", command=self.refresh_results_window)
-        msgCat = Radiobutton(radio_frame, text="Messages by category", variable=self.box, value="Messages by category", command=self.refresh_results_window)
-        msg = Radiobutton(radio_frame, text="Messages", variable=self.box, value="Messages", command=self.refresh_results_window)
+        report = Radiobutton(radio_frame, text="Report", variable=self.box,
+                             value="Report", command=self.refresh_results_window)
+        rawMet = Radiobutton(radio_frame, text="Raw metrics", variable=self.box,
+                             value="Raw metrics", command=self.refresh_results_window)
+        dup = Radiobutton(radio_frame, text="Duplication", variable=self.box,
+                          value="Duplication", command=self.refresh_results_window)
+        ext = Radiobutton(radio_frame, text="External dependencies",
+                          variable=self.box, value="External dependencies",
+                          command=self.refresh_results_window)
+        stat = Radiobutton(radio_frame, text="Statistics by type",
+                           variable=self.box, value="Statistics by type",
+                           command=self.refresh_results_window)
+        msgCat = Radiobutton(radio_frame, text="Messages by category",
+                             variable=self.box, value="Messages by category",
+                             command=self.refresh_results_window)
+        msg = Radiobutton(radio_frame, text="Messages", variable=self.box,
+                            value="Messages", command=self.refresh_results_window)
         report.select()
         report.grid(column=0, row=0, sticky=W)
         rawMet.grid(column=1, row=0, sticky=W)
@@ -274,7 +319,7 @@ class LintGui:
         self.lbMessages.delete(0, END)
         for msg in self.msgs:
             if (self.msg_type_dict.get(msg[0])()):
-                msg_str = self.convert_to_string(msg)
+                msg_str = convert_to_string(msg)
                 self.lbMessages.insert(END, msg_str)
                 fg_color = COLORS.get(msg_str[:3], 'black')
                 self.lbMessages.itemconfigure(END, fg=fg_color)
@@ -289,13 +334,6 @@ class LintGui:
         except:
             pass
 
-    def convert_to_string(self, msg):
-        """make a string representation of a message"""
-        if (msg[2] != ""):
-            return "(" + msg[0] + ") " + msg[1] + "." + msg[2] + " [" + msg[3] + "]: " + msg[4]
-        else:
-            return "(" + msg[0] + ") " + msg[1] + " [" + msg[3] + "]: " + msg[4]
-
     def process_incoming(self):
         """process the incoming messages from running pylint"""
         while self.msg_queue.qsize():
@@ -310,7 +348,7 @@ class LintGui:
 
                 #displaying msg if message type is selected in check box
                 if (self.msg_type_dict.get(msg[0])()):
-                    msg_str = self.convert_to_string(msg)
+                    msg_str = convert_to_string(msg)
                     self.lbMessages.insert(END, msg_str)
                     fg_color = COLORS.get(msg_str[:3], 'black')
                     self.lbMessages.itemconfigure(END, fg=fg_color)
@@ -417,7 +455,7 @@ class LintGui:
 def lint_thread(module, reporter, gui):
     """thread for pylint"""
     gui.status.text = "processing module(s)"
-    lint_obj = pylint.lint.Run(args=[module], reporter=reporter, exit=False)
+    pylint.lint.Run(args=[module], reporter=reporter, exit=False)
     gui.msg_queue.put("DONE")
 
 
@@ -425,9 +463,10 @@ def Run(args):
     """launch pylint gui from args"""
     if args:
         print 'USAGE: pylint-gui\n launch a simple pylint gui using Tk'
-        return
+        sys.exit(1)
     gui = LintGui()
     gui.mainloop()
+    sys.exit(0)
 
 if __name__ == '__main__':
     Run(sys.argv[1:])
