@@ -36,10 +36,7 @@ class HTMLReporter(BaseReporter):
     def add_message(self, msg_id, location, msg):
         """manage message of different type and in the context of path"""
         module, obj, line, col_offset = location[1:]
-        if self.include_ids:
-            sigle = msg_id
-        else:
-            sigle = msg_id[0]
+        sigle = self.make_sigle(msg_id)
         self.msgs += [sigle, module, obj, str(line), str(col_offset), escape(msg)]
 
     def set_output(self, output=None):
