@@ -41,12 +41,12 @@ except NameError:
 
 try:
     POUETT # don't catch me
-except Exception: # pylint:disable-msg = W0703
+except Exception: # pylint:disable = W0703
     POUETT = 'something'
 
 try:
     POUETTT # don't catch me
-except: # pylint:disable-msg = W0702
+except: # pylint:disable = W0702
     POUETTT = 'something'
 
 print POUET, POUETT, POUETTT
@@ -67,3 +67,19 @@ def if_branch_test(something):
     else:
         print xxx
         xxx = 3
+
+
+def decorator(arg):
+    """Decorator with one argument."""
+    return lambda: list(arg)
+
+
+@decorator(arg=[i * 2 for i in range(15)])
+def func1():
+    """A function with a decorator that contains a listcomp."""
+    pass
+
+@decorator(arg=(i * 2 for i in range(15)))
+def func2():
+    """A function with a decorator that contains a genexpr."""
+    pass
