@@ -29,6 +29,11 @@ class RunTC(TestCase):
     def _runtest(self, args, reporter=None, out=None, code=28):
         if out is None:
             out = StringIO()
+        if args and args[-1].startswith('pylint.lint'):
+            try:
+                import cProfile, pstats
+            except ImportError:
+                code += 1
         try:
             sys.stderr = sys.stdout = out
             try:
@@ -76,6 +81,11 @@ class RunTC(TestCase):
     def test_no_ext_file(self):
         self._runtest([join(HERE, 'input', 'noext')], code=0)
 
+    @tag('smoke')
+    def test_generated_members(self):
+        # XXX dual end quotation since optparse buggily remove one...
+        self._runtest(['--generated-members=objects,DoesNotExist,delay,retry,"[a-zA-Z]+_set{1,2}""', 'pylint.lint'])
+
     @tag('smoke')
     def test_w0704_ignored(self):
         self._runtest([join(HERE, 'input', 'ignore_except_pass_by_default.py')], code=0)
