@@ -1,5 +1,5 @@
 # Copyright (c) 2003-2010 Sylvain Thenault (thenault@gmail.com).
-# Copyright (c) 2003-2010 LOGILAB S.A. (Paris, FRANCE).
+# Copyright (c) 2003-2012 LOGILAB S.A. (Paris, FRANCE).
 # http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This program is free software; you can redistribute it and/or modify it under
@@ -19,7 +19,7 @@ main pylint class
 """
 
 import sys
-from os import linesep
+from warnings import warn
 from os.path import dirname, basename, splitext, exists, isdir, join, normpath
 
 from logilab.common.modutils import modpath_from_file, get_module_files, \
@@ -93,7 +93,7 @@ def category_id(id):
 
 
 class Message:
-    def __init__(self, checker, msgid, msg, descr):
+    def __init__(self, checker, msgid, msg, descr, symbol):
         assert len(msgid) == 5, 'Invalid message id %s' % msgid
         assert msgid[0] in MSG_TYPES, \
                'Bad message type %s in %r' % (msgid[0], msgid)
@@ -101,6 +101,7 @@ class Message:
         self.msg = msg
         self.descr = descr
         self.checker = checker
+        self.symbol = symbol
 
 class MessagesHandlerMixIn:
     """a mix-in class containing all the messages related methods for the main
@@ -110,6 +111,8 @@ class MessagesHandlerMixIn:
     def __init__(self):
         # dictionary of registered messages
         self._messages = {}
+        # dictionary from string symbolic id to Message object.
+        self._messages_by_symbol = {}
         self._msgs_state = {}
         self._module_msgs_state = {} # None
         self._msgs_by_category = {}
@@ -126,14 +129,27 @@ class MessagesHandlerMixIn:
         """
         msgs_dict = checker.msgs
         chkid = None
-        for msgid, (msg, msgdescr) in msgs_dict.items():
+        for msgid, msg_tuple in msgs_dict.iteritems():
+            if len(msg_tuple) == 3:
+                (msg, msgsymbol, msgdescr) = msg_tuple
+                assert msgsymbol not in self._messages_by_symbol, \
+                    'Message symbol %r is already defined' % msgsymbol
+            else:
+                # messages should have a symbol, but for backward compatibility
+                # they may not.
+                (msg, msgdescr) = msg_tuple
+                warn("[pylint 0.26] description of message %s doesn't include "
+                     "a symbolic name" % msgid, DeprecationWarning)
+                msgsymbol = None
             # avoid duplicate / malformed ids
             assert msgid not in self._messages, \
                    'Message id %r is already defined' % msgid
             assert chkid is None or chkid == msgid[1:3], \
                    'Inconsistent checker part in message id %r' % msgid
             chkid = msgid[1:3]
-            self._messages[msgid] = Message(checker, msgid, msg, msgdescr)
+            msg = Message(checker, msgid, msg, msgdescr, msgsymbol)
+            self._messages[msgid] = msg
+            self._messages_by_symbol[msgsymbol] = msg
             self._msgs_by_category.setdefault(msgid[0], []).append(msgid)
 
     def get_message_help(self, msgid, checkerref=False):
@@ -144,10 +160,14 @@ class MessagesHandlerMixIn:
             desc += ' This message belongs to the %s checker.' % \
                    msg.checker.name
         title = msg.msg
+        if msg.symbol:
+            symbol_part = ' (%s)' % msg.symbol
+        else:
+            symbol_part = ''
         if title != '%s':
             title = title.splitlines()[0]
-            return ':%s: *%s*\n%s' % (msg.msgid, title, desc)
-        return ':%s:\n%s' % (msg.msgid, desc)
+            return ':%s%s: *%s*\n%s' % (msg.msgid, symbol_part, title, desc)
+        return ':%s%s:\n%s' % (msg.msgid, symbol_part, desc)
 
     def disable(self, msgid, scope='package', line=None):
         """don't output message of the given id"""
@@ -155,20 +175,20 @@ class MessagesHandlerMixIn:
         # msgid is a category?
         catid = category_id(msgid)
         if catid is not None:
-            for msgid in self._msgs_by_category.get(catid):
-                self.disable(msgid, scope, line)
+            for _msgid in self._msgs_by_category.get(catid):
+                self.disable(_msgid, scope, line)
             return
         # msgid is a checker name?
         if msgid.lower() in self._checkers:
             for checker in self._checkers[msgid.lower()]:
-                for msgid in checker.msgs:
-                    self.disable(msgid, scope, line)
+                for _msgid in checker.msgs:
+                    self.disable(_msgid, scope, line)
             return
         # msgid is report id?
         if msgid.lower().startswith('rp'):
             self.disable_report(msgid)
             return
-        # msgid is a msgid.
+        # msgid is a symbolic or numeric msgid.
         msg = self.check_message_id(msgid)
         if scope == 'module':
             assert line > 0
@@ -183,7 +203,7 @@ class MessagesHandlerMixIn:
             msgs = self._msgs_state
             msgs[msg.msgid] = False
             # sync configuration object
-            self.config.disable_msg = [mid for mid, val in msgs.items()
+            self.config.disable_msg = [mid for mid, val in msgs.iteritems()
                                        if not val]
 
     def enable(self, msgid, scope='package', line=None):
@@ -205,7 +225,7 @@ class MessagesHandlerMixIn:
         if msgid.lower().startswith('rp'):
             self.enable_report(msgid)
             return
-        # msgid is a msgid.
+        # msgid is a symbolic or numeric msgid.
         msg = self.check_message_id(msgid)
         if scope == 'module':
             assert line > 0
@@ -218,10 +238,17 @@ class MessagesHandlerMixIn:
             msgs = self._msgs_state
             msgs[msg.msgid] = True
             # sync configuration object
-            self.config.enable = [mid for mid, val in msgs.items() if val]
+            self.config.enable = [mid for mid, val in msgs.iteritems() if val]
 
     def check_message_id(self, msgid):
-        """raise UnknownMessage if the message id is not defined"""
+        """returns the Message object for this message.
+
+        msgid may be either a numeric or symbolic id.
+
+        Raises UnknownMessage if the message id is not defined.
+        """
+        if msgid in self._messages_by_symbol:
+            return self._messages_by_symbol[msgid]
         msgid = msgid.upper()
         try:
             return self._messages[msgid]
@@ -231,7 +258,11 @@ class MessagesHandlerMixIn:
     def is_message_enabled(self, msgid, line=None):
         """return true if the message associated to the given message id is
         enabled
+
+        msgid may be either a numeric or symbolic message id.
         """
+        if msgid in self._messages_by_symbol:
+            msgid = self._messages_by_symbol[msgid].msgid
         if line is None:
             return self._msgs_state.get(msgid, True)
         try:
@@ -317,7 +348,7 @@ class MessagesHandlerMixIn:
                     by_checker[checker.name] = [list(checker.options_and_values()),
                                                 dict(checker.msgs),
                                                 list(checker.reports)]
-        for checker, (options, msgs, reports) in by_checker.items():
+        for checker, (options, msgs, reports) in by_checker.iteritems():
             prefix = ''
             title = '%s checker' % checker
             print title
@@ -333,7 +364,7 @@ class MessagesHandlerMixIn:
                 title = ('%smessages' % prefix).capitalize()
                 print title
                 print '~' * len(title)
-                for msgid in sort_msgs(msgs.keys()):
+                for msgid in sort_msgs(msgs.iterkeys()):
                     print self.get_message_help(msgid, False)
                 print
             if reports:
@@ -349,7 +380,7 @@ class MessagesHandlerMixIn:
         """output full messages list documentation in ReST format"""
         msgids = []
         for checker in self.get_checkers():
-            for msgid in checker.msgs.keys():
+            for msgid in checker.msgs.iterkeys():
                 msgids.append(msgid)
         msgids.sort()
         for msgid in msgids:
@@ -394,9 +425,6 @@ class ReportsHandlerMixIn:
 
     def make_reports(self, stats, old_stats):
         """render registered reports"""
-        if self.config.files_output:
-            filename = 'pylint_global.' + self.reporter.extension
-            self.reporter.set_output(open(filename, 'w'))
         sect = Section('Report',
                        '%s statements analysed.'% (self.stats['statement']))
         for checker in self._reports:
@@ -410,13 +438,13 @@ class ReportsHandlerMixIn:
                     continue
                 report_sect.report_id = reportid
                 sect.append(report_sect)
-        self.reporter.display_results(sect)
+        return sect
 
     def add_stats(self, **kwargs):
         """add some stats entries to the statistic dictionary
         raise an AssertionError if there is a key conflict
         """
-        for key, value in kwargs.items():
+        for key, value in kwargs.iteritems():
             if key[-1] == '_':
                 key = key[:-1]
             assert key not in self.stats
@@ -528,4 +556,3 @@ class PyLintASTWalker(object):
             self.walk(child)
         for cb in self.leave_events.get(cid, ()):
             cb(astng)
-
